<?php

namespace Drupal\bulk_import_export_interface_translation\Form;

use Drupal\locale\Form\ExportForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Drupal\Core\Language\LanguageInterface;
use Drupal\locale\PoDatabaseReader;
use Symfony\Component\HttpFoundation\File\File;
use Drupal\Component\Gettext\PoStreamWriter;

/**
 * Extends the Locale ExportForm to add a bulk export option.
 */
class BulkExportForm extends ExportForm {

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Build the original form.
    $form = parent::buildForm($form, $form_state);
    // Add the 'Bulk export' submit button.
    $form['actions']['bulk_export'] = [
      '#type' => 'submit',
      '#value' => $this->t('Bulk Export'),
      '#submit' => ['::bulkExportSubmit'],
    ];

    return $form;
  }

  /**
   * Submit handler for the bulk export button.
   *
   * @param array $form
   *   The form structure.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function bulkExportSubmit(array &$form, FormStateInterface $form_state) {
    // Create a temporary directory for storing exported files.
    $temp_dir = 'temporary://translations_export';
    $this->fileSystem->prepareDirectory($temp_dir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);

    $files = [];
    $languages = $this->languageManager->getLanguages();

    // Iterate over all languages and export.
    foreach ($languages as $language) {
      $reader = new PoDatabaseReader();
      $langcode = $language->getId();

      // Skip system language.
      if ($langcode == LanguageInterface::LANGCODE_SYSTEM) {
        continue;
      }

      $reader->setLangcode($langcode);
      $reader->setOptions($form_state->getValue('content_options', []));

      $item = $reader->readItem();
      if (!empty($item)) {
        $filename = $langcode . '.po';
        $uri = $temp_dir . '/' . $filename;

        // Set up the PO file header.
        $header = $reader->getHeader();
        $header->setProjectName($this->config('system.site')->get('name'));
        $header->setLanguageName($language->getName());

        // Write the PO file.
        $writer = new PoStreamWriter();
        $writer->setURI($uri);
        $writer->setHeader($header);
        $writer->open();
        $writer->writeItem($item);
        $writer->writeItems($reader);
        $writer->close();

        $files[] = $uri;
      }
    }

    if (!empty($files)) {
      // Create ZIP archive
      // Create ZIP archive with proper temporary file handling.
      $zip_filename = 'translations.zip';
      $zip_uri = $temp_dir . '/' . $zip_filename;
      $zip_path = $this->fileSystem->realpath($zip_uri);
      if (!$zip_path) {
        $zip_path = $this->fileSystem->realpath($temp_dir) . '/' . $zip_filename;
      }
      $zip = new \ZipArchive();
      if ($zip->open($zip_path, \ZipArchive::CREATE | \ZipArchive::OVERWRITE)) {
        foreach ($files as $file) {
          $real_file_path = $this->fileSystem->realpath($file);
          if ($real_file_path) {
            $zip->addFile($real_file_path, basename($file));
          }
        }
        $zip->close();

        // Create a File object using the real path.
        $zip_file = new File($zip_path);

        // Serve the ZIP file.
        $response = new BinaryFileResponse($zip_file);
        $response->setContentDisposition('attachment', $zip_filename);
        $form_state->setResponse($response);

        // Clean up after sending.
        register_shutdown_function(function () use ($temp_dir) {
            $this->fileSystem->deleteRecursive($temp_dir);
        });
        $this->messenger()->addStatus($this->t('Translations exported successfully.'));
      }
      else {
        $this->messenger()->addError($this->t('Failed to create ZIP archive.'));
      }
    }
    else {
      $this->messenger()->addStatus($this->t('Nothing to export.'));
    }
  }

}
