<?php

namespace Drupal\bulk_import_export_interface_translation\Form;

use Drupal\Component\Utility\Environment;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\language\ConfigurableLanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;

/**
 * Form constructor for the translation import screen.
 *
 * @internal
 */
class ImportForm extends FormBase {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The configurable language manager.
   *
   * @var \Drupal\language\ConfigurableLanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The stream wrapper manager service.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_handler'),
      $container->get('language_manager'),
      $container->get('file_system'),
      $container->get('messenger'),
      $container->get('stream_wrapper_manager')
    );
  }

  /**
   * Constructs a form for language import.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\language\ConfigurableLanguageManagerInterface $language_manager
   *   The configurable language manager.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $stream_wrapper_manager
   *   The stream wrapper manager service.
   */
  public function __construct(
    ModuleHandlerInterface $module_handler,
    ConfigurableLanguageManagerInterface $language_manager,
    FileSystemInterface $file_system,
    MessengerInterface $messenger,
    StreamWrapperManagerInterface $stream_wrapper_manager,
  ) {
    $this->moduleHandler = $module_handler;
    $this->languageManager = $language_manager;
    $this->fileSystem = $file_system;
    $this->messenger = $messenger;
    $this->streamWrapperManager = $stream_wrapper_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'locale_translate_import_form';
  }

  /**
   * Form constructor for the translation import screen.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $validators = [
      'FileExtension' => ['extensions' => 'zip'],
      'FileSizeLimit' => ['fileLimit' => Environment::getUploadMaxSize()],
    ];
    $form['zip_file'] = [
      '#type' => 'file',
      '#title' => $this->t('Translation ZIP file'),
      '#description' => [
        '#theme' => 'file_upload_help',
        '#description' => $this->t('A ZIP file containing .po files for translations.'),
        '#upload_validators' => $validators,
      ],
      '#size' => 50,
      '#upload_validators' => $validators,
      '#attributes' => ['class' => ['file-import-input']],
    ];

    $form['customized'] = [
      '#title' => $this->t('Treat imported strings as custom translations'),
      '#type' => 'checkbox',
    ];
    $form['overwrite_options'] = [
      '#type' => 'container',
      '#tree' => TRUE,
    ];
    $form['overwrite_options']['not_customized'] = [
      '#title' => $this->t('Overwrite non-customized translations'),
      '#type' => 'checkbox',
      '#states' => [
        'checked' => [
          ':input[name="customized"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['overwrite_options']['customized'] = [
      '#title' => $this->t('Overwrite existing customized translations'),
      '#type' => 'checkbox',
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $file = _file_save_upload_from_form($form['zip_file'], $form_state, 0);

    // Ensure we have the file uploaded.
    if (!$file) {
      $form_state->setErrorByName('zip_file', $this->t('ZIP file to import not found.'));
    }
    else {
      $form_state->set('uploaded_file', $file);
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->moduleHandler->loadInclude('locale', 'translation.inc');

    $file = $form_state->get('uploaded_file');
    $zip_path = $this->fileSystem->realpath($file->getFileUri());

    $zip = new \ZipArchive();
    if ($zip->open($zip_path) === TRUE) {
      $imported_languages = [];
      for ($i = 0; $i < $zip->numFiles; $i++) {
        $filename = $zip->getNameIndex($i);
        if (preg_match('/^([a-z]{2,3}(-[A-Z]{2})?)\.po$/', $filename, $matches)) {
          $langcode = $matches[1];
          if ($this->languageManager->getLanguage($langcode)) {
            $po_content = $zip->getFromIndex($i);
            $this->importTranslation($langcode, $po_content, $form_state);
            $imported_languages[] = $langcode;
          }
        }
      }
      $zip->close();

      if (!empty($imported_languages)) {
        $this->messenger->addStatus($this->t('Translations imported for languages: @languages', ['@languages' => implode(', ', $imported_languages)]));
      }
      else {
        $this->messenger->addWarning($this->t('No translations were imported.'));
      }
    }
    else {
      $this->messenger->addError($this->t('Unable to open the ZIP file.'));
    }

    $form_state->setRedirect('locale.translate_page');
  }

  /**
   * Imports a translation for a specific language.
   *
   * @param string $langcode
   *   The language code.
   * @param string $po_content
   *   The content of the .po file.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  protected function importTranslation($langcode, $po_content, FormStateInterface $form_state) {
    $options = array_merge(_locale_translation_default_update_options(), [
      'langcode' => $langcode,
      'overwrite_options' => $form_state->getValue('overwrite_options'),
      'customized' => $form_state->getValue('customized') ? LOCALE_CUSTOMIZED : LOCALE_NOT_CUSTOMIZED,
    ]);

    // Create a temporary file for the .po content.
    $temp_file = $this->fileSystem->tempnam('temporary://', 'po_') . ".po";
    $this->fileSystem->saveData($po_content, $temp_file, FileSystemInterface::EXISTS_REPLACE);
    $file = (object) [
      'filename' => str_replace('temporary://', '', $temp_file),
      'uri' => $temp_file,
      'project' => '',
      'version' => '',
      'langcode' => $langcode,
    ];
    $this->moduleHandler->loadInclude('locale', 'bulk.inc');
    $file = locale_translate_file_attach_properties((object) $file, $options);
    $batch = locale_translate_batch_build([$file->uri => $file], $options);
    batch_set($batch);

    // Create or update all configuration translations for this language.
    if ($batch = locale_config_batch_update_components($options, [$langcode])) {
      batch_set($batch);
    }
  }

}
